﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace invoice_manager
{
    /// <summary>
    /// Trida reprezentujici osobu. 
    /// Může se jednat o odběratele nebo dodavatele.
    /// </summary>
    public class Person
    {
        //ICO - Identifikacni cislo organizace
        private int companyId;

        private String name;
        private String surname;
        //číslo účtu
        private String bankAccount;
        
        //DIC - Danove identifikacni cislo, je-li vyplneno jedna se o platce DPH
        private String taxId;

        private String street;
        private int nr;
        //PSC - Postovni smerovaci cislo
        private int postCode;


        //konstruktory
        public Person(int companyId)
        {
            this.companyId = companyId;
        }

        public Person(String name, String surname, int companyId)
            : this(name, surname, companyId, null, null, 0, 0, null)
        {
        }

        public Person(String name, String surname, int companyId, String bankAccount)
            : this(name, surname, companyId, bankAccount, null, 0, 0, null)
        {
        }

        public Person(String name, String surname, int companyId, String bankAccount, String street, int nr, int postCode)
            : this(name, surname, companyId, bankAccount, street, nr, postCode, null)
        {
        }

        /// <summary>
        /// Konstruktor ocekavajici jméno, příjmení, IČO, bankovní účet, ulici, číslo popisné, PSČ a DIČ
        /// Parametry <paramref name="name">name</paramref>, <paramref name="surname">surname</paramref>, 
        /// <paramref name="bankAccount">bankAccount</paramref>, <paramref name="street">street</paramref> a  
        /// <paramref name="taxId">taxId</paramref> jsou typu <see cref="String"/>
        /// Parametry <paramref name="companyId">companyId</paramref>, 
        /// <paramref name="nr">nr</paramref> a <paramref name="postCode">postCode</paramref>
        /// jsou typu <see cref="Int32"/>
        /// </summary>
        /// <param name="name">Jmeno osoby</param>
        /// <param name="surname">Prijmeni osoby</param>
        /// <param name="companyId">IČO</param>
        /// <param name="bankAccount">Bankovní účet</param>
        /// <param name="street">Ulice</param>
        /// <param name="nr">Číslo popisné</param>
        /// <param name="postCode">PSČ</param>
        /// <param name="taxId">DIČ</param>
        public Person(String name, String surname, int companyId, String bankAccount, String street, int nr, int postCode, String taxId)
        {
            this.name = name;
            this.surname = surname;
            this.companyId = companyId;
            this.bankAccount = bankAccount;
            this.street = street;
            this.nr = nr;
            this.postCode = postCode;
            this.taxId = taxId;
        }

        //getry a setry

        /// <value>
        /// Jméno osoby
        /// </value>
        public String Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <value>
        /// Příjmení osoby
        /// </value>
        public String Surname
        {
            get { return surname; }
            set { surname = value; }
        }

        /// <value>
        /// IČO (Identifikační číslo organizace)
        /// </value>
        public int CompanyId
        {
            get { return companyId; }
            set { companyId = value; }
        }

        /// <value>
        /// Číslo bankovního účtu
        /// </value>
        public String BankAccount
        {
            get { return bankAccount; }
            set { bankAccount = value; }
        }

        /// <value>
        /// DIČ (Daňové identifikační číslo) - slouží pro plátce DPH
        /// </value>
        public String TaxId
        {
            get { return taxId; }
            set { taxId = value; }
        }

        /// <value>
        /// Ulice
        /// </value>
        public String Street
        {
            get { return street; }
            set { street = value; }
        }

        /// <value>
        /// Číslo popisné
        /// </value>
        public int NR
        {
            get { return nr; }
            set { nr = value; }
        }

        /// <value>
        /// PSČ (Poštovní směrovací číslo)
        /// </value>
        public int PostCode
        {
            get { return postCode; }
            set { postCode = value; }
        }


        //metody
        public int[] sepBankAccount()
        {
            return sepBankAccount(this.bankAccount);
        }

        public int[] sepBankAccount(String input)
        {
            char[] sep = { '-', '/'};
            String[] tmp = input.Split(sep);
            int[] account = new int[tmp.Length];
            int i = 0;
            foreach (String s in tmp)
            {
                try
                {
                    account[i++] = Convert.ToInt32(s);
                }
                catch (FormatException e)
                {
                    Console.WriteLine("{0}: Input string is not a sequence of digits.", e);
                }
                catch (OverflowException e)
                {
                    Console.WriteLine("{0}: The number cannot fit in an Int32.", e);
                }
            }
            return account;
        }

        public string toSave()
        {
            return "" + name + "|" + surname + "|" + companyId + "|" + bankAccount + "|" + street + "|" + nr + "|" + postCode+ "|" + taxId;
        }

        public override string ToString()
        {
            return "Person: " + companyId + " " + name + " " + surname;
        }

        public override bool Equals(object obj)
        {
            if (this.GetHashCode() == obj.GetHashCode())
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public override int GetHashCode()
        {
            return this.companyId;
        }


    }
}
