﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace invoice_manager
{
    class PersonMenu
    {
        public static void menu()
        {
            Dictionary<char, MenuItem> submenu = new Dictionary<char, MenuItem>();
            submenu.Add('1', new MenuItem() { fce = new fce(createPerson), desc = "Vytvořit" });
            submenu.Add('2', new MenuItem() { fce = new fce(writeAllPerson), desc = "Vypsat" });
            submenu.Add('3', new MenuItem() { fce = new fce(editPersonById), desc = "Upravit" });
            submenu.Add('4', new MenuItem() { fce = new fce(findPersonById), desc = "Najít podle IČ" });
            submenu.Add('5', new MenuItem() { fce = new fce(findPersonBySurname), desc = "Najít podle Příjmení" });
            submenu.Add('6', new MenuItem() { fce = new fce(delPersonById), desc = "Smazat podle IČ" });
            submenu.Add('7', new MenuItem() { fce = new fce(delPersonBySurname), desc = "Smazat podle Příjmení" });
            submenu.Add('*', new MenuItem() { fce = new fce(Program.Return), desc = "Return" });
            submenu.Add('0', new MenuItem() { fce = new fce(Program.Exit), desc = "Konec" });

            while (true)
            {
                foreach (KeyValuePair<char, MenuItem> polozka in submenu)
                {
                    Console.WriteLine("[" + polozka.Key + "] - " + polozka.Value.desc);
                }
                string volba = Console.ReadLine();
                if (submenu.ContainsKey(volba[0]))
                    submenu[volba[0]].fce();
            }
        }

        public static void createPerson()
        {
            String s;
            Person newPerson;

            bool contain, wrongId = false;
            do
            {
                Console.WriteLine("Zadejte IČO: ");
                s = Console.ReadLine();
                int i;
                wrongId = (i = toInt(s)) == int.MinValue ?  true : false;
                if (wrongId)
                {
                    Console.WriteLine("Špatně zadané IČO.");
                }

                newPerson = new Person(i);
                contain = Program.people.Contains(newPerson);
                if (contain)
                {
                    Console.WriteLine("Osoba s tímto IČO již existuje.");
                }
            } while (String.IsNullOrEmpty(s) || contain || wrongId);

             
             

            do
            {
                Console.WriteLine("Zadej jméno: ");
                s = Console.ReadLine();
                newPerson.Name = s;
            } while(String.IsNullOrEmpty(s));

            do
            {
                Console.WriteLine("Zadej příjmení: ");
                s = Console.ReadLine();
                newPerson.Surname = s;
            } while (String.IsNullOrEmpty(s));

            //TO-DO: Načítání zbylých parametrů

            if (!Program.people.Contains(newPerson))
            {
                Program.people.Add(newPerson);
            }
        }

        public static void writeAllPerson()
        {
            Console.WriteLine("---------------------------------------");
            if (Program.people.Count == 0)
                Console.WriteLine("Žádné osoby v seznamu.");
            else    
                Program.people.ForEach(Console.WriteLine);
            Console.WriteLine("---------------------------------------");
        }

        public static void findPersonById()
        {
            Console.WriteLine("Zadejte IČO hledané osoby:");
            int i = toInt(Console.ReadLine());
            Person result = findById(i);
            if (result != null)
            {
                Console.WriteLine(result.ToString());   
            }
            else
            {
                Console.WriteLine("Osoba s IČ {0} nenalezena.", i);
            }
        }

        public static void editPersonById()
        {
            Console.WriteLine("Zadejte IČ osoby kterou chcete upravit:");
            int i = toInt(Console.ReadLine());
            Person result = findById(i);
            if (result != null)
            {
                //TO-DO: Dodělat možnost upravit
                Console.WriteLine("Možnost úprav zatím není dostupná.");
            }
            else
            {
                Console.WriteLine("Osoba s IČ {0} nenalezena.", i);
            }
        }

        public static void findPersonBySurname()
        {
            Console.WriteLine("Zadejte příjmení hledané osoby:");
            String s = Console.ReadLine();
            List<Person> results = Program.people.FindAll(
            delegate(Person p)
            {
                return p.Surname.Equals(s, StringComparison.InvariantCultureIgnoreCase);
            }
            );

            if (results.Count != 0)
            {
                Console.WriteLine("---------------------------------------");
                Console.WriteLine("Počet nalezených osob: {0}", results.Count);
                foreach (Person p in results)
                {
                    Console.WriteLine(p.ToString());
                }
                Console.WriteLine("---------------------------------------");
            }
            else
            {
                Console.WriteLine("Osoba {0} nenalezena.", s);
            }
        }

        public static void delPersonById()
        {
            Console.WriteLine("Zadejte IČ osoby kterou chcete smazat:");
            int i = toInt(Console.ReadLine());
            Person result = findById(i);
            if (result != null)
            {
                if (Program.people.Remove(result))
                {
                    Console.WriteLine("Osoba {0} smazána.", result.ToString());
                }
            }
            else
            {
                Console.WriteLine("Osoba s IČ {0} nenalezena.", i);
            }
            
        }

        public static void delPersonBySurname()
        {
            Console.WriteLine("Zadejte IČ osoby kterou chcete smazat:");
            String s = Console.ReadLine();
            Person result = Program.people.Find(
            delegate(Person p)
            {
                return p.Surname.Equals(s, StringComparison.InvariantCultureIgnoreCase);
            });
            if (result != null)
            {
                if (Program.people.Remove(result))
                {
                    Console.WriteLine("Osoba {0} smazána.", result.ToString());
                }
            }
            else
            {
                Console.WriteLine("Osoba {0} nenalezena.", s);
            }

        }

        public static int toInt(String s)
        {
            int i = int.MinValue;
            try
            {
                i = Convert.ToInt32(s);
            }
            catch (FormatException e)
            {
                Console.WriteLine("Error: Input string is not a sequence of digits.");
            }
            catch (OverflowException e)
            {
                Console.WriteLine("Error: The number cannot fit in an Int32.");
            }
            return i;
        }

        public static Person findById(int id)
        {
            Person finded = Program.people.Find(
            delegate(Person p)
            {
                return p.CompanyId == id;
            }
            );
            return finded;
        }

    }
}
