﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Globalization;

namespace invoice_manager
{
    /// <summary>
    /// Třída pro načtení uložených dat ze souboru
    /// </summary>
    public class DataLoader
    {
        //String pro načtení jedné řádky
        private String line;
        //reader
        StreamReader sr;

        /// <summary>
        /// Metoda načte uložené osoby (Person)
        /// <paramref name="file">String file</paramref>
        /// <param name="file">zdrojový soubor</param>
        /// </summary>
        public List<Person> loadPeople(String file)
        {
            List<Person> people = new List<Person>();
            try
            {
                sr = new StreamReader(file);
                Console.WriteLine("Načítám data ze souboru {0}", file);
                while ((line = sr.ReadLine()) != null)
                {
                    String[] s = line.Split('|');
                    people.Add(new Person(s[0], s[1], toInt(s[2]), s[3], s[4], toInt(s[5]), toInt(s[6]), s[7]));
                }
                Console.WriteLine("Data byla načtena.");
            }
            catch (Exception e)
            {
                Console.WriteLine("Soubor {0} nenalezen nebo není co číst.", file);
                Console.WriteLine(e.Message);
            }
            sr.Close();
            return people;
        }

        /// <summary>
        /// Metoda načte uložené položky (Item)
        /// <paramref name="file">String file</paramref>
        /// <param name="file">zdrojový soubor</param>
        /// </summary>
        public List<Item> loadItems(String file)
        {
            List<Item> items = new List<Item>();
            try
            {
                sr = new StreamReader(file);
                Console.WriteLine("Načítám data ze souboru {0}", file);
                while ((line = sr.ReadLine()) != null)
                {
                    String[] s = line.Split('|');
                    items.Add(new Item(toInt(s[0]), s[1], toInt(s[2]), toInt(s[3])));
                }
                Console.WriteLine("Data byla načtena.");
            }
            catch (Exception e)
            {
                Console.WriteLine("Soubor {0} nenalezen nebo není co číst.", file);
                Console.WriteLine(e.Message);
            }
            sr.Close();
            return items;
        }

        /// <summary>
        /// Metoda načte uložené faktury (Invoice). 
        /// A to jak přijaté tak i vydané
        /// <paramref name="file">String file</paramref>
        /// <paramref name="people">List&lt;Person> people</paramref>
        /// <param name="file">zdrojový soubor</param>
        /// <param name="people">seznam osob</param>
        /// </summary>
        public List<Invoice> loadInvoices(String file, List<Person> people)
        {
            List<Invoice> invoices = new List<Invoice>();
            try
            {
                sr = new StreamReader(file);
                Console.WriteLine("Načítám data ze souboru {0}", file);
                while ((line = sr.ReadLine()) != null)
                {
                    String[] s = line.Split('|');
                    Person subscriber = findPersonById(toInt(s[2]), people);
                    Person supplier = findPersonById(toInt(s[3]), people);
                    
                    //převede datum ze stringu do formátu DateTime
                    DateTimeFormatInfo formatInfo = new DateTimeFormatInfo();
                    formatInfo.ShortDatePattern = "dd-MM-yyyy";
                    formatInfo.DateSeparator = ".";
                    DateTime date = Convert.ToDateTime(s[6], formatInfo);

                    invoices.Add(new Invoice(toInt(s[0]), s[1], subscriber, supplier, toInt(s[4]), toBoolean(s[5]), date));
                }
                Console.WriteLine("Data byla načtena.");
            }
            catch (Exception e)
            {
                Console.WriteLine("Soubor {0} nenalezen nebo není co číst.", file);
                Console.WriteLine(e.Message);
            }
            sr.Close();
            return invoices;
        }

        /// <summary>
        /// Metoda převeda řetězec (String) na číslo (Int32)
        /// <paramref name="s">String s</paramref>
        /// <param name="s">číslo uložené jako řetězec</param>
        /// </summary>
        public static int toInt(String s)
        {
            int i = int.MinValue;
            try
            {
                i = Convert.ToInt32(s);
            }
            catch (FormatException e)
            {
                Console.WriteLine("Error: Input string is not a sequence of digits.");
            }
            catch (OverflowException e)
            {
                Console.WriteLine("Error: The number cannot fit in an Int32.");
            }
            return i;
        }

        /// <summary>
        /// Metoda převeda řetězec (String) na logickou hodnotu (Boolean)
        /// <paramref name="value">String value</paramref>
        /// <param name="value">logická hodnota uložená jako řetězec</param>
        /// </summary>
        private static bool toBoolean(string value)
        {
            bool b = false;
            try
            {
                b = Convert.ToBoolean(value);
            }
            catch (FormatException)
            {
                Console.WriteLine("Unable to convert '{0}' to a Boolean.", value);
            }
            return b;
        }

        /// <summary>
        /// Metoda najde osobu (Person) podle jejího IČ/ID
        /// <paramref name="id">int id</paramref>
        /// <paramref name="people">List<Person> people</paramref>
        /// <param name="id">IČ/ID osoby</param>
        /// <param name="people">seznam všech osob</param>
        /// </summary>
        public static Person findPersonById(int id, List<Person> people)
        {
            Person finded = people.Find(
            delegate(Person p)
            {
                return p.CompanyId == id;
            }
            );
            return finded;
        }
    }
}
